const helpers = require('../../helpers/helpers')
const teamHelper = require('../../helpers/Team')
const mailService = require('pubsweet-component-mail-service')
const logger = require('@pubsweet/logger')
const config = require('config')
const userHelper = require('../../helpers/User')

const statuses = config.get('statuses')
module.exports = models => async (req, res) => {
  // const reqUser = await models.User.find(req.user)
  // TO DO: authsome
  const { collectionId, invitationId } = req.params
  try {
    const collection = await models.Collection.find(collectionId)
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined) {
      res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
      return
    }
    const team = await teamHelper.getTeamByGroupAndCollection(
      collectionId,
      invitation.role,
      models.Team,
    )

    collection.invitations = collection.invitations.filter(
      inv => inv.id !== invitation.id,
    )
    if (invitation.role === 'handlingEditor') {
      collection.status = 'submitted'
      collection.visibleStatus = statuses[collection.status].private
      delete collection.handlingEditor
    } else if (invitation.role === 'reviewer') {
      const reviewerInvitations = collection.invitations.filter(
        inv => inv.role === 'reviewer',
      )
      if (reviewerInvitations.length === 0) {
        collection.status = 'heAssigned'
        collection.visibleStatus = statuses[collection.status].private
      }
    }
    await collection.save()
    await teamHelper.removeTeamMember(team.id, invitation.userId, models.Team)
    const user = await models.User.find(invitation.userId)
    user.teams = user.teams.filter(userTeamId => team.id !== userTeamId)
    await user.save()
    try {
      if (invitation.role === 'handlingEditor') {
        await mailService.setupRevokeInvitationEmail(
          user.email,
          'revoke-handling-editor',
        )
      } else if (invitation.role === 'reviewer') {
        await userHelper.setupReviewerUnassignEmail(
          models,
          collection,
          user,
          mailService,
        )
      }

      return res.status(200).json({})
    } catch (e) {
      logger.error(e.message)
      return res.status(500).json({ error: 'Email could not be sent.' })
    }
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
