const logger = require('@pubsweet/logger')
const helpers = require('../../helpers/helpers')
const teamHelper = require('../../helpers/Team')
const mailService = require('pubsweet-component-mail-service')
const userHelper = require('../../helpers/User')
const collectionHelper = require('../../helpers/Collection')

module.exports = models => async (req, res) => {
  const { collectionId, invitationId } = req.params
  const { isAccepted, reason } = req.body

  if (!helpers.checkForUndefinedParams(isAccepted)) {
    res.status(400).json({ error: 'Missing parameters' })
    logger.error('some parameters are missing')
    return
  }

  let user = await models.User.find(req.user)
  try {
    const collection = await models.Collection.find(collectionId)
    const invitation = await collection.invitations.find(
      invitation => invitation.id === invitationId,
    )
    if (invitation === undefined)
      return res.status(404).json({
        error: `Invitation ${invitationId} not found`,
      })
    if (invitation.hasAnswer)
      return res
        .status(400)
        .json({ error: `${invitation.id} has already been answered` })
    if (invitation.userId !== user.id)
      return res.status(403).json({
        error: `User ${user.email} is not allowed to modify invitation ${
          invitation.id
        }`,
      })

    if (invitation.role === 'handlingEditor')
      await collectionHelper.updateHandlingEditor(collection, isAccepted)
    invitation.timestamp = Date.now()
    invitation.hasAnswer = true
    const eic = await userHelper.getEditorInChief(models.User)
    let toEmail = eic.email
    if (invitation.role === 'reviewer')
      toEmail = collection.handlingEditor.email
    if (isAccepted === true) {
      invitation.isAccepted = true
      await collection.save()
      try {
        await mailService.setupAgreeEmail(
          toEmail,
          user,
          invitation.role,
          `${req.protocol}://${req.get('host')}`,
          collection.customId,
        )
      } catch (e) {
        logger.error(e)
        return res.status(500).json({ error: 'Email could not be sent.' })
      }
    } else {
      invitation.isAccepted = false
      const team = await teamHelper.getTeamByGroupAndCollection(
        collectionId,
        invitation.role,
        models.Team,
      )
      if (invitation.role === 'handlingEditor') {
        delete collection.handlingEditor
      }
      await collection.save()
      await teamHelper.removeTeamMember(team.id, user.id, models.Team)
      user.teams = user.teams.filter(userTeamId => team.id !== userTeamId)
      if (reason !== undefined) {
        invitation.reason = reason
      }
      try {
        if (invitation.role === 'handlingEditor') {
          await mailService.setupDeclineEmail(
            toEmail,
            user,
            'handling-editor-declined',
            collection.customId,
            reason,
          )
        } else if (invitation.role === 'reviewer') {
          return await userHelper.setupReviewerDeclinedEmailData(
            models,
            collection,
            req,
            res,
            user,
            mailService,
          )
        }
      } catch (e) {
        logger.error(e)
        return res.status(500).json({ error: 'Email could not be sent.' })
      }
    }
    user = await user.save()
    res.status(200).json(user)
    return
  } catch (e) {
    const notFoundError = await helpers.handleNotFoundError(e, 'collection')
    return res.status(notFoundError.status).json({
      error: notFoundError.message,
    })
  }
}
